@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,7 +5,7 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /* Lefteris Koutsofios - AT&T Labs Research */
@@ -17,6 +14,8 @@
 #include "g.h"
 #include "gcommon.h"
 #include "mem.h"
+#include <math.h>
+#include <string.h>
 
 #define WCU widget->u.c
 #define WINDOW widget->u.c->window
@@ -466,7 +465,7 @@ int GCarrow (Gwidget_t *widget, Gpoint_t gp1, Gpoint_t gp2, Ggattr_t *ap) {
     if (pd.x == 0 && pd.y == 0)
         return 0;
     tangent = atan2 ((double) pd.y, (double) pd.x);
-    if ((l = sqrt ((double) (pd.x * pd.x + pd.y * pd.y))) > 30)
+    if ((l = hypot(pd.x, pd.y)) > 30)
         l = 30;
     pa.x = l * cos (tangent + M_PI / 7) + pp2.x;
     pa.y = l * sin (tangent + M_PI / 7) + pp2.y;
@@ -764,7 +763,7 @@ static HFONT findfont (char *name, int size) {
     HFONT font;
     int fi;
 
-    if (name[0] == '\000')
+    if (strcmp(name, "") == 0)
         return Gfontp[0].font;
 
     sprintf (&Gbufp[0], name, size);
