@@ -1,28 +1,32 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
- * Copyright (c) 2011 AT&T Intellectual Property 
+ * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include "render.h"
-#include "agxbuf.h"
-#include "htmltable.h"
-#include "entities.h"
-#include "logic.h"
-#include "gvc.h"
+#include <common/render.h>
+#include <cgraph/agxbuf.h>
+#include <common/htmltable.h>
+#include <common/entities.h>
+#include <common/logic.h>
+#include <math.h>
+#include <gvc/gvc.h>
+#include <cgraph/strcasecmp.h>
+#include <stddef.h>
+#include <stdbool.h>
 
 #ifdef _WIN32
-#include "libltdl/lt_system.h"
+#define R_OK 4
 #endif
 
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#endif // HAVE_UNISTD_H
+
 #include <ctype.h>
 
 /*
@@ -102,17 +106,17 @@ double late_double(void *obj, attrsym_t * attr, double def, double low)
  * Return value for PSinputscale. If this is > 0, it has been set on the
  * command line and this value is used.
  * Otherwise, we check the graph's inputscale attribute. If this is not set
- * or has a bad value, we return -1. 
+ * or has a bad value, we return -1.
  * If the value is 0, we return the default. Otherwise, we return the value.
  * Set but negative values are treated like 0.
- */ 
+ */
 double get_inputscale (graph_t* g)
 {
     double d;
 
     if (PSinputscale > 0) return PSinputscale;  /* command line flag prevails */
     d = late_double(g, agfindgraphattr(g, "inputscale"), -1, 0);
-    if (d == 0) return POINTS_PER_INCH; 
+    if (d == 0) return POINTS_PER_INCH;
     else return d;
 }
 
@@ -164,6 +168,9 @@ node_t *UF_union(node_t * u, node_t * v)
 	ND_UF_size(v) = 1;
     } else
 	v = UF_find(v);
+	/* if we have two copies of the same node, their union is just that node */
+    if (u == v)
+	return u;
     if (ND_id(u) > ND_id(v)) {
 	ND_UF_parent(u) = v;
 	ND_UF_size(v) += ND_UF_size(u);
@@ -175,13 +182,6 @@ node_t *UF_union(node_t * u, node_t * v)
     return v;
 }
 
-void UF_remove(node_t * u, node_t * v)
-{
-    assert(ND_UF_size(u) == 1);
-    ND_UF_parent(u) = u;
-    ND_UF_size(v) -= ND_UF_size(u);
-}
-
 void UF_singleton(node_t * u)
 {
     ND_UF_size(u) = 1;
@@ -209,11 +209,11 @@ pointf coord(node_t * n)
 #define W_DEGREE 5
 
 /*
- *  Bezier : 
+ *  Bezier :
  *	Evaluate a Bezier curve at a particular parameter value
  *      Fill in control points for resulting sub-curves if "Left" and
  *	"Right" are non-null.
- * 
+ *
  */
 pointf Bezier(pointf * V, int degree, double t, pointf * Left, pointf * Right)
 {
@@ -260,28 +260,22 @@ Agraphinfo_t* GD_info(graph_t * g) { return ((Agraphinfo_t*)AGDATA(g));}
 Agnodeinfo_t* ND_info(node_t * n) { return ((Agnodeinfo_t*)AGDATA(n));}
 #endif
 
-#if !defined(MSWIN32) && !defined(_WIN32)
+#if !defined(_WIN32)
 #include	<pwd.h>
 
-#if 0
-static void cleanup(void)
-{
-    agxbfree(&xb);
-}
-#endif
 #endif
 
 /* Fgets:
  * Read a complete line.
- * Return pointer to line, 
+ * Return pointer to line,
  * or 0 on EOF
  */
 char *Fgets(FILE * fp)
 {
-    static int bsize = 0;
+    static size_t bsize = 0;
     static char *buf;
     char *lp;
-    int len;
+    size_t len;
 
     len = 0;
     do {
@@ -289,7 +283,7 @@ char *Fgets(FILE * fp)
 	    bsize += BUFSIZ;
 	    buf = grealloc(buf, bsize);
 	}
-	lp = fgets(buf + len, bsize - len, fp);
+	lp = fgets(buf + len, (int)(bsize - len), fp);
 	if (lp == 0)
 	    break;
 	len += strlen(lp);	/* since lp != NULL, len > 0 */
@@ -305,23 +299,23 @@ char *Fgets(FILE * fp)
  * Check to make sure it is okay to read in files.
  * It returns NULL if the filename is trivial.
  *
- * If the application has set the SERVER_NAME environment variable, 
+ * If the application has set the SERVER_NAME environment variable,
  * this indicates it is web-active. In this case, it requires that the GV_FILE_PATH
  * environment variable be set. This gives the legal directories
  * from which files may be read. safefile then derives the rightmost component
- * of filename, where components are separated by a slash, backslash or colon, 
- * It then checks for the existence of a file consisting of a directory from 
+ * of filename, where components are separated by a slash, backslash or colon,
+ * It then checks for the existence of a file consisting of a directory from
  * GV_FILE_PATH followed by the rightmost component of filename. It returns the
  * first such found, or NULL otherwise.
  * The filename returned is thus
  * Gvfilepath concatenated with the last component of filename,
  * where a component is determined by a slash, backslash or colon
- * character.  
+ * character.
  *
  * If filename contains multiple components, the user is
  * warned, once, that everything to the left is ignored.
  *
- * For non-server applications, we use the path list in Gvimagepath to 
+ * For non-server applications, we use the path list in Gvimagepath to
  * resolve relative pathnames.
  *
  * N.B. safefile uses a fixed buffer, so functions using it should use the
@@ -333,13 +327,13 @@ char *Fgets(FILE * fp)
 #define PATHSEP ":"
 #endif
 
-static char** mkDirlist (const char* list, int* maxdirlen)
+static char** mkDirlist (const char* list, size_t* maxdirlen)
 {
     int cnt = 0;
     char* s = strdup (list);
     char* dir;
     char** dirs = NULL;
-    int maxlen = 0;
+    size_t maxlen = 0;
 
     for (dir = strtok (s, PATHSEP); dir; dir = strtok (NULL, PATHSEP)) {
 	dirs = ALLOC (cnt+2,dirs,char*);
@@ -351,7 +345,7 @@ static char** mkDirlist (const char* list, int* maxdirlen)
     return dirs;
 }
 
-static char* findPath (char** dirs, int maxdirlen, const char* str)
+static char* findPath (char** dirs, size_t maxdirlen, const char* str)
 {
     static char *safefilename = NULL;
     char** dp;
@@ -360,7 +354,7 @@ static char* findPath (char** dirs, int maxdirlen, const char* str)
          * +1 for null character.
          * +1 for directory separator character.
          */
-    safefilename = realloc(safefilename, (maxdirlen + strlen(str) + 2));
+    safefilename = realloc(safefilename, maxdirlen + strlen(str) + 2);
 
     for (dp = dirs; *dp; dp++) {
 	sprintf (safefilename, "%s%s%s", *dp, DIRSEP, str);
@@ -374,7 +368,7 @@ const char *safefile(const char *filename)
 {
     static boolean onetime = TRUE;
     static char *pathlist = NULL;
-    static int maxdirlen;
+    static size_t maxdirlen;
     static char** dirs;
     const char *str, *p;
 
@@ -382,7 +376,7 @@ const char *safefile(const char *filename)
 	return NULL;
 
     if (HTTPServerEnVar) {   /* If used as a server */
-	/* 
+	/*
 	 * If we are running in an http server we allow
 	 * files only from the directory specified in
 	 * the GV_FILE_PATH environment variable.
@@ -541,20 +535,11 @@ pointf spline_at_y(splines * spl, double y)
 /* this caching seems to prevent p.x from getting set from bz.list[0].x
 	- optimizer problem ? */
 
-#if 0
-    static splines *mem = NULL;
-
-    if (mem != spl) {
-	mem = spl;
-#endif
 	for (i = 0; i < spl->size; i++) {
 	    bz = spl->list[i];
 	    if (BETWEEN(bz.list[bz.size - 1].y, y, bz.list[0].y))
 		break;
 	}
-#if 0
-    }
-#endif
     if (y > bz.list[0].y)
 	p = bz.list[0];
     else if (y < bz.list[bz.size - 1].y)
@@ -584,7 +569,7 @@ pointf spline_at_y(splines * spl, double y)
 	    t = (low + high) / 2.0;
 	    p = Bezier(c, 3, t, NULL, NULL);
 	    d = p.y - y;
-	    if (ABS(d) <= 1)
+	    if (fabs(d) <= 1)
 		break;
 	    if (d < 0)
 		high = t;
@@ -596,18 +581,11 @@ pointf spline_at_y(splines * spl, double y)
     return p;
 }
 
-pointf neato_closest(splines * spl, pointf p)
-{
-/* this is a stub so that we can share a common emit.c between dot and neato */
-
-    return spline_at_y(spl, p.y);
-}
-
 static int Tflag;
 void gvToggle(int s)
 {
     Tflag = !Tflag;
-#if !defined(MSWIN32) && !defined(_WIN32)
+#if !defined(_WIN32)
     signal(SIGUSR1, gvToggle);
 #endif
 }
@@ -637,11 +615,11 @@ void common_init_node(node_t * n)
     fi.fontsize = late_double(n, N_fontsize, DEFAULT_FONTSIZE, MIN_FONTSIZE);
     fi.fontname = late_nnstring(n, N_fontname, DEFAULT_FONTNAME);
     fi.fontcolor = late_nnstring(n, N_fontcolor, DEFAULT_COLOR);
-    ND_label(n) = make_label((void*)n, str,
+    ND_label(n) = make_label(n, str,
 	        ((aghtmlstr(str) ? LT_HTML : LT_NONE) | ( (shapeOf(n) == SH_RECORD) ? LT_RECD : LT_NONE)),
 		fi.fontsize, fi.fontname, fi.fontcolor);
     if (N_xlabel && (str = agxget(n, N_xlabel)) && (str[0])) {
-	ND_xlabel(n) = make_label((void*)n, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
+	ND_xlabel(n) = make_label(n, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(agraphof(n)) |= NODE_XLABEL;
     }
@@ -671,16 +649,16 @@ initFontLabelEdgeAttr(edge_t * e, struct fontinfo *fi,
  * Return true if head/tail end of edge should not be clipped
  * to node.
  */
-static boolean 
+static bool
 noClip(edge_t *e, attrsym_t* sym)
 {
     char		*str;
-    boolean		rv = FALSE;
+    bool rv = false;
 
     if (sym) {	/* mapbool isn't a good fit, because we want "" to mean true */
 	str = agxget(e,sym);
 	if (str && str[0]) rv = !mapbool(str);
-	else rv = FALSE;
+	else rv = false;
     }
     return rv;
 }
@@ -689,7 +667,7 @@ noClip(edge_t *e, attrsym_t* sym)
  */
 static port
 chkPort (port (*pf)(node_t*, char*, char*), node_t* n, char* s)
-{ 
+{
     port pt;
 	char* cp=NULL;
 	if(s)
@@ -721,7 +699,7 @@ int common_init_edge(edge_t * e)
     if (E_label && (str = agxget(e, E_label)) && (str[0])) {
 	r = 1;
 	initFontEdgeAttr(e, &fi);
-	ED_label(e) = make_label((void*)e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
+	ED_label(e) = make_label(e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(sg) |= EDGE_LABEL;
 	ED_label_ontop(e) =
@@ -731,7 +709,7 @@ int common_init_edge(edge_t * e)
     if (E_xlabel && (str = agxget(e, E_xlabel)) && (str[0])) {
 	if (!fi.fontname)
 	    initFontEdgeAttr(e, &fi);
-	ED_xlabel(e) = make_label((void*)e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
+	ED_xlabel(e) = make_label(e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
 				fi.fontsize, fi.fontname, fi.fontcolor);
 	GD_has_labels(sg) |= EDGE_XLABEL;
     }
@@ -740,21 +718,21 @@ int common_init_edge(edge_t * e)
     /* vladimir */
     if (E_headlabel && (str = agxget(e, E_headlabel)) && (str[0])) {
 	initFontLabelEdgeAttr(e, &fi, &lfi);
-	ED_head_label(e) = make_label((void*)e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
+	ED_head_label(e) = make_label(e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
 				lfi.fontsize, lfi.fontname, lfi.fontcolor);
 	GD_has_labels(sg) |= HEAD_LABEL;
     }
     if (E_taillabel && (str = agxget(e, E_taillabel)) && (str[0])) {
 	if (!lfi.fontname)
 	    initFontLabelEdgeAttr(e, &fi, &lfi);
-	ED_tail_label(e) = make_label((void*)e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
+	ED_tail_label(e) = make_label(e, str, (aghtmlstr(str) ? LT_HTML : LT_NONE),
 				lfi.fontsize, lfi.fontname, lfi.fontcolor);
 	GD_has_labels(sg) |= TAIL_LABEL;
     }
     /* end vladimir */
 
-    /* We still accept ports beginning with colons but this is deprecated 
-     * That is, we allow tailport = ":abc" as well as the preferred 
+    /* We still accept ports beginning with colons but this is deprecated
+     * That is, we allow tailport = ":abc" as well as the preferred
      * tailport = "abc".
      */
     str = agget(e, TAIL_ID);
@@ -911,7 +889,7 @@ void compute_bb(graph_t * g)
 
 int is_a_cluster (Agraph_t* g)
 {
-    return ((g == g->root) || (!strncasecmp(agnameof(g), "cluster", 7)));
+    return ((g == g->root) || (!strncasecmp(agnameof(g), "cluster", 7)) || mapBool(agget(g,"cluster"),FALSE));
 }
 
 /* setAttr:
@@ -949,28 +927,20 @@ static node_t *clustNode(node_t * n, graph_t * cg, agxbuf * xb,
 {
     node_t *cn;
     static int idx = 0;
-    char num[100];
 
-    agxbput(xb, "__");
-    sprintf(num, "%d", idx++);
-    agxbput(xb, num);
-    agxbputc(xb, ':');
-    agxbput(xb, agnameof(cg));
+    agxbprint(xb, "__%d:%s", idx++, agnameof(cg));
 
     cn = agnode(agroot(cg), agxbuse(xb), 1);
     agbindrec(cn, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
 
     SET_CLUST_NODE(cn);
 	agsubnode(cg,cn,1);
-	//aginsert(cg, cn);
 	agsubnode(clg,n,1);
-	//aginsert(clg, n);
 
     /* set attributes */
     N_label = setAttr(agraphof(cn), cn, "label", "", N_label);
     N_style = setAttr(agraphof(cn), cn, "style", "invis", N_style);
     N_shape = setAttr(agraphof(cn), cn, "shape", "box", N_shape);
-    /* N_width = setAttr (cn->graph, cn, "width", "0.0001", N_width); */
 
     return cn;
 }
@@ -1042,6 +1012,7 @@ static edge_t *cloneEdge(edge_t * e, node_t * ct, node_t * ch)
     edge_t *ce = agedge(g, ct, ch,NULL,1);
     agbindrec(ce, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
     agcopyattr(e, ce);
+    ED_compound(ce) = TRUE;
 
     return ce;
 }
@@ -1075,11 +1046,11 @@ static item *mapEdge(Dt_t * map, edge_t * e)
 
     key[0] = agtail(e);
     key[1] = aghead(e);
-    return (item *) dtmatch(map, &key);
+    return dtmatch(map, &key);
 }
 
 /* checkCompound:
- * If endpoint names a cluster, mark for temporary deletion and create 
+ * If endpoint names a cluster, mark for temporary deletion and create
  * special node and insert into cluster. Then clone the edge. Real edge
  * will be deleted when we delete the original node.
  * Invariant: new edge has same sense as old. That is, given t->h with
@@ -1091,11 +1062,12 @@ static item *mapEdge(Dt_t * map, edge_t * e)
  * routing to avoid edge crossings. At present, this is not implemented,
  * so we could use a simpler model in which we create a single cluster
  * node for each cluster used in a cluster edge.
+ *
+ * Return 1 if cluster edge is created.
  */
 #define MAPC(n) (strncmp(agnameof(n),"cluster",7)?NULL:findCluster(cmap,agnameof(n)))
 
-
-static void
+static int
 checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 {
     graph_t *tg;
@@ -1107,20 +1079,20 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
     edge_t *ce;
     item *ip;
 
-    if (IS_CLUST_NODE(h)) return;
+    if (IS_CLUST_NODE(h)) return 0;
     tg = MAPC(t);
     hg = MAPC(h);
     if (!tg && !hg)
-	return;
+	return 0;
     if (tg == hg) {
 	agerr(AGWARN, "cluster cycle %s -- %s not supported\n", agnameof(t),
 	      agnameof(t));
-	return;
+	return 0;
     }
     ip = mapEdge(map, e);
     if (ip) {
 	cloneEdge(e, ip->t, ip->h);
-	return;
+	return 1;
     }
 
     if (hg) {
@@ -1128,12 +1100,12 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	    if (agcontains(hg, tg)) {
 		agerr(AGWARN, "tail cluster %s inside head cluster %s\n",
 		      agnameof(tg), agnameof(hg));
-		return;
+		return 0;
 	    }
 	    if (agcontains(tg, hg)) {
 		agerr(AGWARN, "head cluster %s inside tail cluster %s\n",
 		      agnameof(hg),agnameof(tg));
-		return;
+		return 0;
 	    }
 	    cn = clustNode(t, tg, xb, clg);
 	    cn1 = clustNode(h, hg, xb, clg);
@@ -1143,7 +1115,7 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	    if (agcontains(hg, t)) {
 		agerr(AGWARN, "tail node %s inside head cluster %s\n",
 		      agnameof(t), agnameof(hg));
-		return;
+		return 0;
 	    }
 	    cn = clustNode(h, hg, xb, clg);
 	    ce = cloneEdge(e, t, cn);
@@ -1153,23 +1125,40 @@ checkCompound(edge_t * e, graph_t * clg, agxbuf * xb, Dt_t * map, Dt_t* cmap)
 	if (agcontains(tg, h)) {
 	    agerr(AGWARN, "head node %s inside tail cluster %s\n", agnameof(h),
 		  agnameof(tg));
-	    return;
+	    return 0;
 	}
 	cn = clustNode(t, tg, xb, clg);
 	ce = cloneEdge(e, cn, h);
 	insertEdge(map, t, h, ce);
     }
+    return 1;
+}
+
+typedef struct {
+    Agrec_t hdr;
+    int n_cluster_edges;
+} cl_edge_t;
+
+static int
+num_clust_edges(graph_t * g)
+{
+    cl_edge_t* cl_info = (cl_edge_t*)HAS_CLUST_EDGE(g);
+    if (cl_info)
+	return cl_info->n_cluster_edges;
+    else
+	return 0;
 }
 
 /* processClusterEdges:
  * Look for cluster edges. Replace cluster edge endpoints
  * corresponding to a cluster with special cluster nodes.
  * Delete original nodes.
- * Return 0 if no cluster edges; 1 otherwise.
+ * If cluster edges are found, a cl_edge_t record will be
+ * attached to the graph, containing the count of such edges.
  */
-int processClusterEdges(graph_t * g)
+void processClusterEdges(graph_t * g)
 {
-    int rv;
+    int num_cl_edges = 0;
     node_t *n;
     node_t *nxt;
     edge_t *e;
@@ -1186,21 +1175,22 @@ int processClusterEdges(graph_t * g)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (IS_CLUST_NODE(n)) continue;
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    checkCompound(e, clg, &xb, map, cmap);
+	    num_cl_edges += checkCompound(e, clg, &xb, map, cmap);
 	}
     }
     agxbfree(&xb);
     dtclose(map);
-    rv = agnnodes(clg);
     for (n = agfstnode(clg); n; n = nxt) {
 	nxt = agnxtnode(clg, n);
 	agdelete(g, n);
     }
     agclose(clg);
-    if (rv)
-	SET_CLUST_EDGE(g);
+    if (num_cl_edges) {
+	cl_edge_t* cl_info;
+	cl_info = agbindrec(g, CL_EDGE_TAG, sizeof(cl_edge_t), FALSE);
+	cl_info->n_cluster_edges = num_cl_edges;
+    }
     dtclose(cmap);
-    return rv;
 }
 
 /* mapN:
@@ -1228,6 +1218,7 @@ static node_t *mapN(node_t * n, graph_t * clg)
 	return nn;
     nn = agnode(g, name, 1);
     agbindrec(nn, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+    SET_CLUST_NODE(nn);
 
     /* Set all attributes to default */
     for (sym = agnxtattr(g, AGNODE, NULL); sym;  (sym = agnxtattr(g, AGNODE, sym))) {
@@ -1245,8 +1236,6 @@ static void undoCompound(edge_t * e, graph_t * clg)
     node_t *nhead;
     edge_t* ce;
 
-    if (!(IS_CLUST_NODE(t) || IS_CLUST_NODE(h)))
-	return;
     ntail = mapN(t, clg);
     nhead = mapN(h, clg);
     ce = cloneEdge(e, ntail, nhead);
@@ -1276,15 +1265,25 @@ void undoClusterEdges(graph_t * g)
     node_t *nextn;
     edge_t *e;
     graph_t *clg;
+    edge_t **elist;
+    int ecnt = num_clust_edges(g);
+    int i = 0;
 
+    if (!ecnt) return;
     clg = agsubg(g, "__clusternodes",1);
     agbindrec(clg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+    elist = N_NEW(ecnt, edge_t*);
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    undoCompound(e, clg);
+	    if (ED_compound(e))
+		elist[i++] = e;
 	}
     }
-    for (n = agfstnode(clg); n; n = nextn) { 
+    assert(i == ecnt);
+    for (i = 0; i < ecnt; i++)
+	undoCompound(elist[i], clg);
+    free (elist);
+    for (n = agfstnode(clg); n; n = nextn) {
 	nextn = agnxtnode(clg, n);
 	gv_cleanup_node(n);
 	agdelete(g, n);
@@ -1296,18 +1295,18 @@ void undoClusterEdges(graph_t * g)
  * Find the attribute belonging to graph g for objects like obj
  * with given name. If one does not exist, create it with the
  * default value def.
- */ 
+ */
 attrsym_t*
 safe_dcl(graph_t * g, int obj_kind, char *name, char *def)
 {
     attrsym_t *a = agattr(g,obj_kind,name, NULL);
-    if (!a)	/* attribute does not exist */		
+    if (!a)	/* attribute does not exist */
 	a = agattr(g,obj_kind,name,def);
     return a;
 }
 
 static int comp_entities(const void *e1, const void *e2) {
-  return strcmp(((struct entities_s *)e1)->name, ((struct entities_s *)e2)->name);
+  return strcmp(((const struct entities_s *)e1)->name, ((const struct entities_s *)e2)->name);
 }
 
 #define MAXENTLEN 8
@@ -1333,17 +1332,14 @@ char* scanEntity (char* t, agxbuf* xb)
     res = bsearch(&key, entities, NR_OF_ENTITIES,
         sizeof(entities[0]), comp_entities);
     if (!res) return t;
-    sprintf (buf, "%d", res->value);
-    agxbputc(xb, '#');
-    agxbput(xb, buf);
-    agxbputc(xb, ';');
+    agxbprint(xb, "#%d;", res->value);
     return (endp+1);
 }
 
 
 /* htmlEntity:
  * Check for an HTML entity for a special character.
- * Assume *s points to first byte after '&'. 
+ * Assume *s points to first byte after '&'.
  * If successful, return the corresponding value and update s to
  * point after the terminating ';'.
  * On failure, return 0 and leave s unchanged.
@@ -1444,7 +1440,6 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
     static boolean warned;
     char*  ns;
     agxbuf xb;
-    unsigned char buf[BUFSIZ];
     unsigned char c;
     unsigned int v;
 
@@ -1456,7 +1451,7 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
 	warned = 0;
     }
 
-    agxbinit(&xb, BUFSIZ, buf);
+    agxbinit(&xb, 0, NULL);
 
     while ((c = *(unsigned char*)s++)) {
         if (c < 0xC0)
@@ -1490,12 +1485,12 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
 		    if (v < 0x7F) /* entity needs 1 byte in UTF8 */
 			c = v;
 		    else if (v < 0x07FF) { /* entity needs 2 bytes in UTF8 */
-			agxbputc(&xb, (v >> 6) | 0xC0);
+			agxbputc(&xb, (char)((v >> 6) | 0xC0));
 			c = (v & 0x3F) | 0x80;
 		    }
 		    else { /* entity needs 3 bytes in UTF8 */
-			agxbputc(&xb, (v >> 12) | 0xE0);
-			agxbputc(&xb, ((v >> 6) & 0x3F) | 0x80);
+			agxbputc(&xb, (char)((v >> 12) | 0xE0));
+			agxbputc(&xb, (char)(((v >> 6) & 0x3F) | 0x80));
 			c = (v & 0x3F) | 0x80;
 		    }
 		    }
@@ -1503,10 +1498,10 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
         else /* copy n byte UTF8 characters */
             for (ui = 0; ui < uc; ++ui)
                 if ((*s & 0xC0) == 0x80) {
-                    agxbputc(&xb, c);
+                    agxbputc(&xb, (char)c);
                     c = *(unsigned char*)s++;
                 }
-                else { 
+                else {
 		            if (!warned) {
 		                agerr(AGWARN, "Invalid %d-byte UTF8 found in input of graph %s - treated as Latin-1. Perhaps \"-Gcharset=latin1\" is needed?\n", uc + 1, agnameof(g));
 		                warned = 1;
@@ -1514,9 +1509,9 @@ char* htmlEntityUTF8 (char* s, graph_t* g)
 		            c = cvtAndAppend (c, &xb);
                     break;
 	            }
-	    agxbputc(&xb, c);
+        agxbputc(&xb, (char)c);
     }
-    ns = strdup (agxbuse(&xb));
+    ns = agxbdisown(&xb);
     agxbfree(&xb);
     return ns;
 }
@@ -1530,10 +1525,9 @@ char* latin1ToUTF8 (char* s)
 {
     char*  ns;
     agxbuf xb;
-    unsigned char buf[BUFSIZ];
     unsigned int  v;
 
-    agxbinit(&xb, BUFSIZ, buf);
+    agxbinit(&xb, 0, NULL);
 
     /* Values are either a byte (<= 256) or come from htmlEntity, whose
      * values are all less than 0x07FF, so we need at most 3 bytes.
@@ -1544,18 +1538,18 @@ char* latin1ToUTF8 (char* s)
 	    if (!v) v = '&';
         }
 	if (v < 0x7F)
-	    agxbputc(&xb, v);
+	    agxbputc(&xb, (char)v);
 	else if (v < 0x07FF) {
-	    agxbputc(&xb, (v >> 6) | 0xC0);
-	    agxbputc(&xb, (v & 0x3F) | 0x80);
+	    agxbputc(&xb, (char)((v >> 6) | 0xC0));
+	    agxbputc(&xb, (char)((v & 0x3F) | 0x80));
 	}
 	else {
-	    agxbputc(&xb, (v >> 12) | 0xE0);
-	    agxbputc(&xb, ((v >> 6) & 0x3F) | 0x80);
-	    agxbputc(&xb, (v & 0x3F) | 0x80);
+	    agxbputc(&xb, (char)((v >> 12) | 0xE0));
+	    agxbputc(&xb, (char)(((v >> 6) & 0x3F) | 0x80));
+	    agxbputc(&xb, (char)((v & 0x3F) | 0x80));
 	}
     }
-    ns = strdup (agxbuse(&xb));
+    ns = agxbdisown(&xb);
     agxbfree(&xb);
     return ns;
 }
@@ -1570,34 +1564,33 @@ utf8ToLatin1 (char* s)
 {
     char*  ns;
     agxbuf xb;
-    unsigned char buf[BUFSIZ];
     unsigned char c;
     unsigned char outc;
 
-    agxbinit(&xb, BUFSIZ, buf);
+    agxbinit(&xb, 0, NULL);
 
     while ((c = *(unsigned char*)s++)) {
 	if (c < 0x7F)
-	    agxbputc(&xb, c);
+	    agxbputc(&xb, (char)c);
 	else {
 	    outc = (c & 0x03) << 6;
 	    c = *(unsigned char*)s++;
 	    outc = outc | (c & 0x3F);
-	    agxbputc(&xb, outc);
+	    agxbputc(&xb, (char)outc);
 	}
     }
-    ns = strdup (agxbuse(&xb));
+    ns = agxbdisown(&xb);
     agxbfree(&xb);
     return ns;
 }
 
-boolean overlap_node(node_t *n, boxf b)
+bool overlap_node(node_t *n, boxf b)
 {
     inside_t ictxt;
     pointf p;
 
     if (! OVERLAP(b, ND_bb(n)))
-        return FALSE;
+        return false;
 
 /*  FIXME - need to do something better about CLOSEENOUGH */
     p = sub_pointf(ND_coord(n), mid_pointf(b.UR, b.LL));
@@ -1608,7 +1601,7 @@ boolean overlap_node(node_t *n, boxf b)
     return ND_shape(n)->fns->insidefn(&ictxt, p);
 }
 
-boolean overlap_label(textlabel_t *lp, boxf b)
+bool overlap_label(textlabel_t *lp, boxf b)
 {
     pointf s;
     boxf bb;
@@ -1620,16 +1613,13 @@ boolean overlap_label(textlabel_t *lp, boxf b)
     return OVERLAP(b, bb);
 }
 
-static boolean overlap_arrow(pointf p, pointf u, double scale, int flag, boxf b)
+static bool overlap_arrow(pointf p, pointf u, double scale, boxf b)
 {
-    if (OVERLAP(b, arrow_bb(p, u, scale, flag))) {
-	/* FIXME - check inside arrow shape */
-	return TRUE;
-    }
-    return FALSE;
+    // FIXME - check inside arrow shape
+    return OVERLAP(b, arrow_bb(p, u, scale));
 }
 
-static boolean overlap_bezier(bezier bz, boxf b)
+static bool overlap_bezier(bezier bz, boxf b)
 {
     int i;
     pointf p, u;
@@ -1639,23 +1629,23 @@ static boolean overlap_bezier(bezier bz, boxf b)
     for (i = 1; i < bz.size; i++) {
 	p = bz.list[i];
 	if (lineToBox(p, u, b) != -1)
-	    return TRUE;
+	    return true;
 	u = p;
     }
 
     /* check arrows */
     if (bz.sflag) {
-	if (overlap_arrow(bz.sp, bz.list[0], 1, bz.sflag, b))
-	    return TRUE;
+	if (overlap_arrow(bz.sp, bz.list[0], 1, b))
+	    return true;
     }
     if (bz.eflag) {
-	if (overlap_arrow(bz.ep, bz.list[bz.size - 1], 1, bz.eflag, b))
-	    return TRUE;
+	if (overlap_arrow(bz.ep, bz.list[bz.size - 1], 1, b))
+	    return true;
     }
-    return FALSE;
+    return false;
 }
 
-boolean overlap_edge(edge_t *e, boxf b)
+bool overlap_edge(edge_t *e, boxf b)
 {
     int i;
     splines *spl;
@@ -1665,93 +1655,54 @@ boolean overlap_edge(edge_t *e, boxf b)
     if (spl && boxf_overlap(spl->bb, b))
         for (i = 0; i < spl->size; i++)
             if (overlap_bezier(spl->list[i], b))
-                return TRUE;
+                return true;
 
     lp = ED_label(e);
     if (lp && overlap_label(lp, b))
-        return TRUE;
+        return true;
 
-    return FALSE;
+    return false;
 }
 
 /* edgeType:
  * Convert string to edge type.
  */
-int edgeType (char* s, int dflt)
+static int edgeType(const char *s, int dflt)
 {
-    int et;
-
-    if (!s || (*s == '\0')) return dflt;
-
-    et = ET_NONE;
-    switch (*s) {
-    case '0' :    /* false */
-	et = ET_LINE;
-	break;
-    case '1' :    /* true */
-    case '2' :
-    case '3' :
-    case '4' :
-    case '5' :
-    case '6' :
-    case '7' :
-    case '8' :
-    case '9' :
-	et = ET_SPLINE;
-	break;
-    case 'c' :
-    case 'C' :
-	if (!strcasecmp (s+1, "urved"))
-	    et = ET_CURVED;
-	else if (!strcasecmp (s+1, "ompound"))
-	    et = ET_COMPOUND;
-	break;
-    case 'f' :
-    case 'F' :
-	if (!strcasecmp (s+1, "alse"))
-	    et = ET_LINE;
-	break;
-    case 'l' :
-    case 'L' :
-	if (!strcasecmp (s+1, "ine"))
-	    et = ET_LINE;
-	break;
-    case 'n' :
-    case 'N' :
-	if (!strcasecmp (s+1, "one")) return et;
-	if (!strcasecmp (s+1, "o")) return ET_LINE;
-	break;
-    case 'o' :
-    case 'O' :
-	if (!strcasecmp (s+1, "rtho"))
-	    et = ET_ORTHO;
-	break;
-    case 'p' :
-    case 'P' :
-	if (!strcasecmp (s+1, "olyline"))
-	    et = ET_PLINE;
-	break;
-    case 's' :
-    case 'S' :
-	if (!strcasecmp (s+1, "pline"))
-	    et = ET_SPLINE;
-	break;
-    case 't' :
-    case 'T' :
-	if (!strcasecmp (s+1, "rue"))
-	    et = ET_SPLINE;
-	break;
-    case 'y' :
-    case 'Y' :
-	if (!strcasecmp (s+1, "es"))
-	    et = ET_SPLINE;
-	break;
+    if (s == NULL || strcmp(s, "") == 0) {
+	return dflt;
     }
-    if (!et) {
-	agerr(AGWARN, "Unknown \"splines\" value: \"%s\" - ignored\n", s);
-	et = dflt;
+
+    if (*s == '0') { /* false */
+	return EDGETYPE_LINE;
+    } else if (*s >= '1' && *s <= '9') { /* true */
+	return EDGETYPE_SPLINE;
+    } else if (strcasecmp(s, "curved") == 0) {
+	return EDGETYPE_CURVED;
+    } else if (strcasecmp(s, "compound") == 0) {
+	return EDGETYPE_COMPOUND;
+    } else if (strcasecmp(s, "false") == 0) {
+	return EDGETYPE_LINE;
+    } else if (strcasecmp(s, "line") == 0) {
+	return EDGETYPE_LINE;
+    } else if (strcasecmp(s, "none") == 0) {
+	return EDGETYPE_NONE;
+    } else if (strcasecmp(s, "no") == 0) {
+	return EDGETYPE_LINE;
+    } else if (strcasecmp(s, "ortho") == 0) {
+	return EDGETYPE_ORTHO;
+    } else if (strcasecmp(s, "polyline") == 0) {
+	return EDGETYPE_PLINE;
+    } else if (strcasecmp(s, "spline") == 0) {
+	return EDGETYPE_SPLINE;
+    } else if (strcasecmp(s, "true") == 0) {
+	return EDGETYPE_SPLINE;
+    } else if (strcasecmp(s, "yes") == 0) {
+	return EDGETYPE_SPLINE;
     }
-    return et;
+
+    agerr(AGWARN, "Unknown \"splines\" value: \"%s\" - ignored\n", s);
+    return dflt;
 }
 
 /* setEdgeType:
@@ -1759,12 +1710,12 @@ int edgeType (char* s, int dflt)
  * If the attribute is not defined, use default.
  * If the attribute is "", use NONE.
  * If attribute value matches (case indepedent), use match.
- *   ortho => ET_ORTHO
- *   none => ET_NONE
- *   line => ET_LINE
- *   polyline => ET_PLINE
- *   spline => ET_SPLINE
- * If attribute is boolean, true means ET_SPLINE, false means ET_LINE.
+ *   ortho => EDGETYPE_ORTHO
+ *   none => EDGETYPE_NONE
+ *   line => EDGETYPE_LINE
+ *   polyline => EDGETYPE_PLINE
+ *   spline => EDGETYPE_SPLINE
+ * If attribute is boolean, true means EDGETYPE_SPLINE, false means EDGETYPE_LINE.
  * Else warn and use default.
  */
 void setEdgeType (graph_t* g, int dflt)
@@ -1776,7 +1727,7 @@ void setEdgeType (graph_t* g, int dflt)
 	et = dflt;
     }
     else if (*s == '\0') {
-	et = ET_NONE;
+	et = EDGETYPE_NONE;
     }
     else et = edgeType (s, dflt);
     GD_flags(g) |= et;
@@ -1787,19 +1738,18 @@ void setEdgeType (graph_t* g, int dflt)
  * Evaluates the extreme points of an ellipse or polygon
  * Determines the point at the center of the extreme points
  * If isRadial is true,sets the inner radius to half the distance to the min point;
- * else uses the angle parameter to identify two points on a line that defines the 
+ * else uses the angle parameter to identify two points on a line that defines the
  * gradient direction
  * By default, this assumes a left-hand coordinate system (for svg); if RHS = 2 flag
  * is set, use standard coordinate system.
  */
-void get_gradient_points(pointf * A, pointf * G, int n, float angle, int flags)
-{
+void get_gradient_points(pointf *A, pointf *G, int n, double angle, int flags) {
     int i;
     double rx, ry;
     pointf min,max,center;
     int isRadial = flags & 1;
     int isRHS = flags & 2;
-    
+
     if (n == 2) {
       rx = A[1].x - A[0].x;
       ry = A[1].y - A[0].y;
@@ -1807,7 +1757,7 @@ void get_gradient_points(pointf * A, pointf * G, int n, float angle, int flags)
       max.x = A[0].x + rx;
       min.y = A[0].y - ry;
       max.y = A[0].y + ry;
-    }    
+    }
     else {
       min.x = max.x = A[0].x;
       min.y = max.y = A[0].y;
@@ -1853,53 +1803,6 @@ void get_gradient_points(pointf * A, pointf * G, int n, float angle, int flags)
     }
 }
 
-#ifndef WIN32_STATIC
-#ifndef HAVE_STRCASECMP
-
-
-#include <string.h>
-//#include <ctype.h>
-
-
-int strcasecmp(const char *s1, const char *s2)
-{
-    while ((*s1 != '\0')
-	   && (tolower(*(unsigned char *) s1) ==
-	       tolower(*(unsigned char *) s2))) {
-	s1++;
-	s2++;
-    }
-
-    return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
-}
-
-#endif				/* HAVE_STRCASECMP */
-#endif				/* WIN32_STATIC */
-
-#ifndef WIN32_STATIC
-#ifndef HAVE_STRNCASECMP
-#include <string.h>
-//#include <ctype.h>
-
-int strncasecmp(const char *s1, const char *s2, unsigned int n)
-{
-    if (n == 0)
-	return 0;
-
-    while ((n-- != 0)
-	   && (tolower(*(unsigned char *) s1) ==
-	       tolower(*(unsigned char *) s2))) {
-	if (n == 0 || *s1 == '\0' || *s2 == '\0')
-	    return 0;
-	s1++;
-	s2++;
-    }
-
-    return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
-}
-
-#endif				/* HAVE_STRNCASECMP */
-#endif                          /* WIN32_STATIC */
 void gv_free_splines(edge_t * e)
 {
     int i;
@@ -1921,18 +1824,18 @@ void gv_cleanup_edge(edge_t * e)
     free_label(ED_head_label(e));
     free_label(ED_tail_label(e));
 	/*FIX HERE , shallow cleaning may not be enough here */
-	agdelrec(e, "Agedgeinfo_t");	
+	agdelrec(e, "Agedgeinfo_t");
 }
 
 void gv_cleanup_node(node_t * n)
 {
-    if (ND_pos(n)) free(ND_pos(n));
+    free(ND_pos(n));
     if (ND_shape(n))
         ND_shape(n)->fns->freefn(n);
     free_label(ND_label(n));
     free_label(ND_xlabel(n));
 	/*FIX HERE , shallow cleaning may not be enough here */
-	agdelrec(n, "Agnodeinfo_t");	
+	agdelrec(n, "Agnodeinfo_t");
 }
 
 void gv_nodesize(node_t * n, boolean flip)
@@ -1943,7 +1846,7 @@ void gv_nodesize(node_t * n, boolean flip)
         w = INCH2PS(ND_height(n));
         ND_lw(n) = ND_rw(n) = w / 2;
         ND_ht(n) = INCH2PS(ND_width(n));
-    } 
+    }
     else {
         w = INCH2PS(ND_width(n));
         ND_lw(n) = ND_rw(n) = w / 2;
@@ -1951,24 +1854,6 @@ void gv_nodesize(node_t * n, boolean flip)
     }
 }
 
-#ifdef _WIN32	
-void fix_fc(void)
-{
-    char buf[28192];
-    char buf2[28192];
-    int cur=0;
-    FILE* fp;
-
-    if((fp = fopen("fix-fc.exe", "r")) == NULL)
-	    return ;
-    fclose (fp);
-    if (!system ("fix-fc.exe")) {
-	system ("del fix-fc.exe");
-	system ("dot -c");	//run dot -c once too since we already run things :)
-    }
-}
-#endif
-
 #ifndef HAVE_DRAND48
 double drand48(void)
 {
@@ -2034,7 +1919,7 @@ Dt_t* mkClustMap (Agraph_t* g)
     Dt_t* map = dtopen (&strDisc, Dtoset);
 
     fillMap (g, map);
-    
+
     return map;
 }
 
@@ -2047,8 +1932,3 @@ findCluster (Dt_t* map, char* name)
     else
 	return NULL;
 }
-
-Agnodeinfo_t* ninf(Agnode_t* n) {return (Agnodeinfo_t*)AGDATA(n);}
-Agraphinfo_t* ginf(Agraph_t* g) {return (Agraphinfo_t*)AGDATA(g);}
-Agedgeinfo_t* einf(Agedge_t* e) {return (Agedgeinfo_t*)AGDATA(e);}
-/* void dumpG(Agraph_t* g) { agwrite(g, stderr); } */
