@@ -10,13 +10,13 @@
  * This version is released under the CPL (Common Public License) with
  * the Graphviz distribution.
  * A version is also available under the LGPL as part of the Adaptagrams
- * project: http://sourceforge.net/projects/adaptagrams.  
+ * project: https://github.com/mjwybrow/adaptagrams.  
  * If you make improvements or bug fixes to this code it would be much
  * appreciated if you could also contribute those changes back to the
  * Adaptagrams repository.
  */
 
-#include "constraint.h"
+#include <vpsc/constraint.h>
 #include <cassert>
 Constraint::Constraint(Variable *left, Variable *right, double gap, bool equality)
 : left(left),
@@ -43,10 +43,6 @@ Constraint::~Constraint() {
 }
 std::ostream& operator <<(std::ostream &os, const Constraint &c)
 {
-	if(&c==NULL) {
-		os<<"NULL";
-	} else {
-		os<<*c.left<<"+"<<c.gap<<"<="<<*c.right<<"("<<c.slack()<<")"<<(c.active?"-active":"");
-	}
+	os<<*c.left<<"+"<<c.gap<<"<="<<*c.right<<"("<<c.slack()<<")"<<(c.active?"-active":"");
 	return os;
 }
